@@ -28,13 +28,13 @@
 
 from __future__ import absolute_import, unicode_literals, division
 
-import codecs
 import copy
 import json
 import math
 import os
 import sys
 import unicodedata
+import warnings
 from collections import deque, namedtuple, defaultdict
 
 from wcwidth import wcwidth
@@ -63,14 +63,12 @@ Savepoint = namedtuple("Savepoint", [
     "g0_charset",
     "g1_charset",
     "charset",
-    "use_utf8",
     "origin",
     "wrap"
 ])
 
-#: A container for a single character, field names are *hopefully*
-#: self-explanatory.
-_Char = namedtuple("_Char", [
+
+class Char(namedtuple("Char", [
     "data",
     "fg",
     "bg",
@@ -79,18 +77,28 @@ _Char = namedtuple("_Char", [
     "underscore",
     "strikethrough",
     "reverse",
-])
+])):
+    """A single styled on-screen character.
 
-
-class Char(_Char):
-    """A wrapper around :class:`_Char`, providing some useful defaults
-    for most of the attributes.
+    :param str data: unicode character. Invariant: ``len(data) == 1``.
+    :param str fg: foreground colour. Defaults to ``"default"``.
+    :param str bg: background colour. Defaults to ``"default"``.
+    :param bool bold: flag for rendering the character using bold font.
+                      Defaults to ``False``.
+    :param bool italics: flag for rendering the character using italic font.
+                         Defaults to ``False``.
+    :param bool underline: flag for rendering the character underlined.
+                           Defaults to ``False``.
+    :param bool strikethrough: flag for rendering the character with a
+                               strike-through line. Defaults to ``False``.
+    :param bool reverse: flag for swapping foreground and background colours
+                         during rendering. Defaults to ``False``.
     """
     __slots__ = ()
 
     def __new__(cls, data, fg="default", bg="default", bold=False,
-                italics=False, underscore=False, reverse=False,
-                strikethrough=False):
+                italics=False, underscore=False,
+                strikethrough=False, reverse=False):
         return super(Char, cls).__new__(cls, data, fg, bg, bold, italics,
                                         underscore, strikethrough, reverse)
 
@@ -143,6 +151,19 @@ class Screen(object):
 
        A sparse ``lines x columns`` :class:`~pyte.screens.Char` matrix.
 
+    .. attribute:: dirty
+
+       A set of line numbers, which should be re-drawn. The user is responsible
+       for clearing this set when changes have been applied.
+
+       >>> screen = Screen(80, 24)
+       >>> screen.dirty.clear()
+       >>> screen.draw("!")
+       >>> list(screen.dirty)
+       [0]
+
+       .. versionadded:: 0.7.0
+
     .. attribute:: cursor
 
        Reference to the :class:`~pyte.screens.Cursor` object, holding
@@ -150,8 +171,12 @@ class Screen(object):
 
     .. attribute:: margins
 
-       Top and bottom screen margins, defining the scrolling region;
-       the actual values are top and bottom line.
+       Margins determine which screen lines move during scrolling
+       (see :meth:`index` and :meth:`reverse_index`). Characters added
+       outside the scrolling region do not make the screen to scroll.
+
+       The value is ``None`` if margins are set to screen boundaries,
+       otherwise -- a pair 0-based top and bottom line indices.
 
     .. attribute:: charset
 
@@ -168,7 +193,7 @@ class Screen(object):
     .. warning::
 
        :data:`~pyte.modes.LNM` is reset by default, to match VT220
-       specification. Unfortunatelly this makes :mode:`pyte` fail
+       specification. Unfortunatelly this makes :mod:`pyte` fail
        ``vttest`` for cursor movement.
 
     .. versionchanged:: 0.4.8
@@ -185,8 +210,7 @@ class Screen(object):
        for a description of the presentational component, implemented
        by ``Screen``.
     """
-    #: A plain empty character with default foreground and background
-    #: colors.
+    #: An empty character with default foreground and background colors.
     default_char = Char(data=" ", fg="default", bg="default")
 
     def __init__(self, columns, lines):
@@ -194,6 +218,7 @@ class Screen(object):
         self.columns = columns
         self.lines = lines
         self.buffer = defaultdict(lambda: StaticDefaultDict(self.default_char))
+        self.dirty = set()
         self.reset()
 
     def __repr__(self):
@@ -219,12 +244,13 @@ class Screen(object):
     def reset(self):
         """Reset the terminal to its initial state.
 
-        * Scroll margins are reset to screen boundaries.
+        * Scrolling margins are reset to screen boundaries.
         * Cursor is moved to home location -- ``(0, 0)`` and its
           attributes are set to defaults (see :attr:`default_char`).
         * Screen is cleared -- each character is reset to
           :attr:`default_char`.
         * Tabstops are reset to "every eight columns".
+        * All lines are marked as :attr:`dirty`.
 
         .. note::
 
@@ -232,9 +258,11 @@ class Screen(object):
            and tabstops should be reset as well, thanks to
            :manpage:`xterm` -- we now know that.
         """
+        self.dirty.update(range(self.lines))
         self.buffer.clear()
+        self.margins = None
+
         self.mode = set([mo.DECAWM, mo.DECTCEM])
-        self.margins = Margins(0, self.lines - 1)
 
         self.title = ""
         self.icon_name = ""
@@ -242,13 +270,11 @@ class Screen(object):
         self.charset = 0
         self.g0_charset = cs.LAT1_MAP
         self.g1_charset = cs.VT100_MAP
-        self.use_utf8 = True
-        self.utf8_decoder = codecs.getincrementaldecoder("utf-8")("replace")
 
         # From ``man terminfo`` -- "... hardware tabs are initially
         # set every `n` spaces when the terminal is powered up. Since
         # we aim to support VT102 / VT220 and linux -- we use n = 8.
-        self.tabstops = set(range(7, self.columns, 8))
+        self.tabstops = set(range(8, self.columns, 8))
 
         self.cursor = Cursor(0, 0)
         self.cursor_position()
@@ -264,68 +290,69 @@ class Screen(object):
         be added at the right, and if it has more -- columns will be
         clipped at the right.
 
-        .. note:: According to `xterm`, we should also reset origin
-                  mode and screen margins, see ``xterm/screen.c:1761``.
-
         :param int lines: number of lines in the new screen.
         :param int columns: number of columns in the new screen.
+
+        .. versionchanged:: 0.7.0
+
+           If the requested screen size is identical to the current screen
+           size, the method does nothing.
         """
         lines = lines or self.lines
         columns = columns or self.columns
 
-        # First resize the lines:
-        diff = self.lines - lines
+        if lines == self.lines and columns == self.columns:
+            return  # No changes.
 
-        # if the current display size is greater than requested
-        #    size, take lines off the top.
-        if diff > 0:
+        self.dirty.update(range(lines))
+
+        if lines < self.lines:
             self.save_cursor()
             self.cursor_position(0, 0)
-            self.delete_lines(diff)
+            self.delete_lines(self.lines - lines)  # Drop from the top.
             self.restore_cursor()
 
-        # Then resize the columns:
-        diff = self.columns - columns
-
-        # if the current display size is greater than requested
-        #    size, trim each line from the right to the new size.
-        if diff > 0:
+        if columns < self.columns:
             for line in self.buffer.values():
-                for x in range(self.columns - diff, self.columns):
+                for x in range(columns, self.columns):
                     line.pop(x, None)
 
         self.lines, self.columns = lines, columns
         self.set_margins()
-        self.reset_mode(mo.DECOM)
 
     def set_margins(self, top=None, bottom=None):
         """Select top and bottom margins for the scrolling region.
 
-        Margins determine which screen lines move during scrolling
-        (see :meth:`index` and :meth:`reverse_index`). Characters added
-        outside the scrolling region do not cause the screen to scroll.
-
         :param int top: the smallest line number that is scrolled.
         :param int bottom: the biggest line number that is scrolled.
         """
-        if top is None or bottom is None:
-            self.margins = Margins(0, self.lines - 1)
+        if top is None and bottom is None:
+            self.margins = None
+            return
+
+        margins = self.margins or Margins(0, self.lines - 1)
+
+        # Arguments are 1-based, while :attr:`margins` are zero
+        # based -- so we have to decrement them by one. We also
+        # make sure that both of them is bounded by [0, lines - 1].
+        if top is None:
+            top = margins.top
         else:
-            # Arguments are 1-based, while :attr:`margins` are zero
-            # based -- so we have to decrement them by one. We also
-            # make sure that both of them is bounded by [0, lines - 1].
             top = max(0, min(top - 1, self.lines - 1))
+        if bottom is None:
+            bottom = margins.bottom
+        else:
             bottom = max(0, min(bottom - 1, self.lines - 1))
 
-            # Even though VT102 and VT220 require DECSTBM to ignore
-            # regions of width less than 2, some programs (like aptitude
-            # for example) rely on it. Practicality beats purity.
-            if bottom - top >= 1:
-                self.margins = Margins(top, bottom)
+        # Even though VT102 and VT220 require DECSTBM to ignore
+        # regions of width less than 2, some programs (like aptitude
+        # for example) rely on it. Practicality beats purity.
+        if bottom - top >= 1:
+            self.margins = Margins(top, bottom)
 
-                # The cursor moves to the home position when the top and
-                # bottom margins of the scrolling region (DECSTBM) changes.
-                self.cursor_position()
+            # The cursor moves to the home position when the top and
+            # bottom margins of the scrolling region (DECSTBM) changes.
+            self.cursor_position()
 
     def set_mode(self, *modes, **kwargs):
         """Set (enable) a given list of modes.
@@ -337,6 +364,8 @@ class Screen(object):
         # private ones.
         if kwargs.get("private"):
             modes = [mode << 5 for mode in modes]
+            if mo.DECSCNM in modes:
+                self.dirty.update(range(self.lines))
 
         self.mode.update(modes)
 
@@ -347,8 +376,7 @@ class Screen(object):
             self.erase_in_display(2)
             self.cursor_position()
 
-        # According to `vttest`, DECOM should also home the cursor, see
-        # vttest/main.c:303.
+        # According to VT520 manual, DECOM should also home the cursor.
         if mo.DECOM in modes:
             self.cursor_position()
 
@@ -374,6 +402,8 @@ class Screen(object):
         # private ones.
         if kwargs.get("private"):
             modes = [mode << 5 for mode in modes]
+            if mo.DECSCNM in modes:
+                self.dirty.update(range(self.lines))
 
         self.mode.difference_update(modes)
 
@@ -445,6 +475,7 @@ class Screen(object):
             # entered.
             if self.cursor.x == self.columns:
                 if mo.DECAWM in self.mode:
+                    self.dirty.add(self.cursor.y)
                     self.carriage_return()
                     self.linefeed()
                 elif char_width > 0:
@@ -463,7 +494,8 @@ class Screen(object):
                 # A two-cell character has a stub slot after it.
                 line[self.cursor.x] = self.cursor.attrs._replace(data=char)
                 if self.cursor.x + 1 < self.columns:
-                    line[self.cursor.x + 1] = self.cursor.attrs._replace(data=" ")
+                    line[self.cursor.x + 1] = self.cursor.attrs \
+                        ._replace(data="")
             elif char_width == 0 and unicodedata.combining(char):
                 # A zero-cell character is combined with the previous
                 # character either on this or preceeding line.
@@ -477,13 +509,15 @@ class Screen(object):
                     self.buffer[self.cursor.y - 1][self.columns - 1] = \
                         last._replace(data=normalized)
             else:
-                pass  # Unprintable character or doesn't advance the cursor.
+                break  # Unprintable character or doesn't advance the cursor.
 
             # .. note:: We can't use :meth:`cursor_forward()`, because that
             #           way, we'll never know when to linefeed.
             if char_width > 0:
                 self.cursor.x = min(self.cursor.x + char_width, self.columns)
 
+        self.dirty.add(self.cursor.y)
+
     def set_title(self, param):
         """Set terminal title.
 
@@ -506,11 +540,12 @@ class Screen(object):
         """Move the cursor down one line in the same column. If the
         cursor is at the last line, create a new line at the bottom.
         """
-        top, bottom = self.margins
-
+        top, bottom = self.margins or Margins(0, self.lines - 1)
         if self.cursor.y == bottom:
-            for line in range(top, bottom):
-                self.buffer[line] = self.buffer[line + 1]
+            # TODO: mark only the lines within margins?
+            self.dirty.update(range(self.lines))
+            for y in range(top, bottom):
+                self.buffer[y] = self.buffer[y + 1]
             self.buffer.pop(bottom, None)
         else:
             self.cursor_down()
@@ -519,11 +554,12 @@ class Screen(object):
         """Move the cursor up one line in the same column. If the cursor
         is at the first line, create a new line at the top.
         """
-        top, bottom = self.margins
-
+        top, bottom = self.margins or Margins(0, self.lines - 1)
         if self.cursor.y == top:
-            for line in range(bottom, top, -1):
-                self.buffer[line] = self.buffer[line - 1]
+            # TODO: mark only the lines within margins?
+            self.dirty.update(range(self.lines))
+            for y in range(bottom, top, -1):
+                self.buffer[y] = self.buffer[y - 1]
             self.buffer.pop(top, None)
         else:
             self.cursor_up()
@@ -562,7 +598,6 @@ class Screen(object):
                                          self.g0_charset,
                                          self.g1_charset,
                                          self.charset,
-                                         self.use_utf8,
                                          mo.DECOM in self.mode,
                                          mo.DECAWM in self.mode))
 
@@ -576,7 +611,6 @@ class Screen(object):
             self.g0_charset = savepoint.g0_charset
             self.g1_charset = savepoint.g1_charset
             self.charset = savepoint.charset
-            self.use_utf8 = savepoint.use_utf8
 
             if savepoint.origin:
                 self.set_mode(mo.DECOM)
@@ -600,10 +634,11 @@ class Screen(object):
         :param count: number of lines to insert.
         """
         count = count or 1
-        top, bottom = self.margins
+        top, bottom = self.margins or Margins(0, self.lines - 1)
 
         # If cursor is outside scrolling margins it -- do nothin'.
         if top <= self.cursor.y <= bottom:
+            self.dirty.update(range(self.cursor.y, self.lines))
             for y in range(bottom, self.cursor.y - 1, -1):
                 if y + count <= bottom and y in self.buffer:
                     self.buffer[y + count] = self.buffer[y]
@@ -620,10 +655,11 @@ class Screen(object):
         :param int count: number of lines to delete.
         """
         count = count or 1
-        top, bottom = self.margins
+        top, bottom = self.margins or Margins(0, self.lines - 1)
 
         # If cursor is outside scrolling margins -- do nothin'.
         if top <= self.cursor.y <= bottom:
+            self.dirty.update(range(self.cursor.y, self.lines))
             for y in range(self.cursor.y, bottom + 1):
                 if y + count <= bottom:
                     if y + count in self.buffer:
@@ -641,6 +677,8 @@ class Screen(object):
 
         :param int count: number of characters to insert.
         """
+        self.dirty.add(self.cursor.y)
+
         count = count or 1
         line = self.buffer[self.cursor.y]
         for x in range(self.columns, self.cursor.x - 1, -1):
@@ -656,7 +694,9 @@ class Screen(object):
 
         :param int count: number of characters to delete.
         """
+        self.dirty.add(self.cursor.y)
         count = count or 1
+
         line = self.buffer[self.cursor.y]
         for x in range(self.cursor.x, self.columns):
             if x + count <= self.columns:
@@ -671,22 +711,26 @@ class Screen(object):
 
         :param int count: number of characters to erase.
 
-        .. warning::
+        .. note::
 
-           Even though *ALL* of the VTXXX manuals state that character
-           attributes **should be reset to defaults**, ``libvte``,
-           ``xterm`` and ``ROTE`` completely ignore this. Same applies
-           too all ``erase_*()`` and ``delete_*()`` methods.
+           Using cursor attributes for character attributes may seem
+           illogical, but if recall that a terminal emulator emulates
+           a type writer, it starts to make sense. The only way a type
+           writer could erase a character is by typing over it.
         """
+        self.dirty.add(self.cursor.y)
         count = count or 1
 
+        line = self.buffer[self.cursor.y]
         for x in range(self.cursor.x,
                        min(self.cursor.x + count, self.columns)):
-            self.buffer[self.cursor.y][x] = self.cursor.attrs
+            line[x] = self.cursor.attrs
 
     def erase_in_line(self, how=0, private=False):
         """Erase a line in a specific way.
 
+        Character attributes are set to cursor attributes.
+
         :param int how: defines the way the line should be erased in:
 
             * ``0`` -- Erases from cursor to end of line, including cursor
@@ -694,19 +738,15 @@ class Screen(object):
             * ``1`` -- Erases from beginning of line to cursor,
               including cursor position.
             * ``2`` -- Erases complete line.
-        :param bool private: when ``True`` character attributes are left
-                             unchanged **not implemented**.
+        :param bool private: when ``True`` only characters marked as
+                             eraseable are affected **not implemented**.
         """
+        self.dirty.add(self.cursor.y)
         if how == 0:
-            # a) erase from the cursor to the end of line, including
-            #    the cursor,
             interval = range(self.cursor.x, self.columns)
         elif how == 1:
-            # b) erase from the beginning of the line to the cursor,
-            #    including it,
             interval = range(self.cursor.x + 1)
         elif how == 2:
-            # c) erase the entire line.
             interval = range(self.columns)
 
         line = self.buffer[self.cursor.y]
@@ -716,6 +756,8 @@ class Screen(object):
     def erase_in_display(self, how=0, private=False):
         """Erases display in a specific way.
 
+        Character attributes are set to cursor attributes.
+
         :param int how: defines the way the line should be erased in:
 
             * ``0`` -- Erases from cursor to end of screen, including
@@ -725,27 +767,22 @@ class Screen(object):
             * ``2`` and ``3`` -- Erases complete display. All lines
               are erased and changed to single-width. Cursor does not
               move.
-        :param bool private: when ``True`` character attributes are left
-                             unchanged **not implemented**.
+        :param bool private: when ``True`` only characters marked as
+                             eraseable are affected **not implemented**.
         """
         if how == 0:
-            # a) erase from cursor to the end of the display, including
-            #    the cursor,
             interval = range(self.cursor.y + 1, self.lines)
         elif how == 1:
-            # b) erase from the beginning of the display to the cursor,
-            #    including it,
             interval = range(self.cursor.y)
         elif how == 2 or how == 3:
-            # c) erase the whole display.
             interval = range(self.lines)
 
+        self.dirty.update(interval)
         for y in interval:
             line = self.buffer[y]
             for x in line:
                 line[x] = self.cursor.attrs
 
-        # In case of 0 or 1 we have to erase the line with the cursor.
         if how == 0 or how == 1:
             self.erase_in_line(how)
 
@@ -781,7 +818,7 @@ class Screen(object):
                                  cursor is bounded by top and and bottom
                                  margins, instead of ``[0; lines - 1]``.
         """
-        if use_margins or mo.DECOM in self.mode:
+        if (use_margins or mo.DECOM in self.mode) and self.margins is not None:
             top, bottom = self.margins
         else:
             top, bottom = 0, self.lines - 1
@@ -794,7 +831,8 @@ class Screen(object):
 
         :param int count: number of lines to skip.
         """
-        self.cursor.y = max(self.cursor.y - (count or 1), self.margins.top)
+        top, _bottom = self.margins or Margins(0, self.lines - 1)
+        self.cursor.y = max(self.cursor.y - (count or 1), top)
 
     def cursor_up1(self, count=None):
         """Move cursor up the indicated # of lines to column 1. Cursor
@@ -811,7 +849,8 @@ class Screen(object):
 
         :param int count: number of lines to skip.
         """
-        self.cursor.y = min(self.cursor.y + (count or 1), self.margins.bottom)
+        _top, bottom = self.margins or Margins(0, self.lines - 1)
+        self.cursor.y = min(self.cursor.y + (count or 1), bottom)
 
     def cursor_down1(self, count=None):
         """Move cursor down the indicated # of lines to column 1.
@@ -860,7 +899,7 @@ class Screen(object):
 
         # If origin mode (DECOM) is set, line number are relative to
         # the top scrolling margin.
-        if mo.DECOM in self.mode:
+        if self.margins is not None and mo.DECOM in self.mode:
             line += self.margins.top
 
             # Cursor is not allowed to move out of the scrolling region.
@@ -904,6 +943,7 @@ class Screen(object):
 
     def alignment_display(self):
         """Fills screen with uppercase E's for screen focus and alignment."""
+        self.dirty.update(range(self.lines))
         for y in range(self.lines):
             for x in range(self.columns):
                 self.buffer[y][x] = self.buffer[y][x]._replace(data="E")
@@ -957,10 +997,15 @@ class Screen(object):
         """Report terminal identity.
 
         .. versionadded:: 0.5.0
+
+        .. versionchanged:: 0.7.0
+
+           If ``private`` keyword argument is set, the method does nothing.
+           This behaviour is consistent with VT220 manual.
         """
         # We only implement "primary" DA which is the only DA request
         # VT102 understood, see ``VT102ID`` in ``linux/drivers/tty/vt.c``.
-        if mode == 0:
+        if mode == 0 and not kwargs.get("private"):
             self.write_process_input(ctrl.CSI + "?6c")
 
     def report_device_status(self, mode):
@@ -1000,105 +1045,31 @@ class Screen(object):
 
 
 class DiffScreen(Screen):
-    """A screen subclass, which maintains a set of dirty lines in its
+    """
+    A screen subclass, which maintains a set of dirty lines in its
     :attr:`dirty` attribute. The end user is responsible for emptying
     a set, when a diff is applied.
 
-    .. attribute:: dirty
+    .. deprecated:: 0.7.0
 
-       A set of line numbers, which should be re-drawn.
-
-       >>> screen = DiffScreen(80, 24)
-       >>> screen.dirty.clear()
-       >>> screen.draw("!")
-       >>> list(screen.dirty)
-       [0]
+       The functionality contained in this class has been merged into
+       :class:`~pyte.screens.Screen` and will be removed in 0.8.0.
+       Please update your code accordingly.
     """
-    def __init__(self, *args):
-        self.dirty = set()
-        super(DiffScreen, self).__init__(*args)
+    def __init__(self, *args, **kwargs):
+        warnings.warn(
+            "The functionality of ``DiffScreen` has been merged into "
+            "``Screen`` and will be removed in 0.8.0. Please update "
+            "your code accordingly.", DeprecationWarning)
 
-    def set_mode(self, *modes, **kwargs):
-        if mo.DECSCNM >> 5 in modes and kwargs.get("private"):
-            self.dirty.update(range(self.lines))
-        super(DiffScreen, self).set_mode(*modes, **kwargs)
-
-    def reset_mode(self, *modes, **kwargs):
-        if mo.DECSCNM >> 5 in modes and kwargs.get("private"):
-            self.dirty.update(range(self.lines))
-        super(DiffScreen, self).reset_mode(*modes, **kwargs)
-
-    def reset(self):
-        self.dirty.update(range(self.lines))
-        super(DiffScreen, self).reset()
-
-    def resize(self, *args, **kwargs):
-        self.dirty.update(range(self.lines))
-        super(DiffScreen, self).resize(*args, **kwargs)
-
-    def draw(self, *args):
-        # Call the superclass's method before marking the row as
-        # dirty, as when wrapping is enabled, draw() might change
-        # self.cursor.y.
-        super(DiffScreen, self).draw(*args)
-        self.dirty.add(self.cursor.y)
-
-    def index(self):
-        if self.cursor.y == self.margins.bottom:
-            self.dirty.update(range(self.lines))
-
-        super(DiffScreen, self).index()
-
-    def reverse_index(self):
-        if self.cursor.y == self.margins.top:
-            self.dirty.update(range(self.lines))
-
-        super(DiffScreen, self).reverse_index()
-
-    def insert_lines(self, *args):
-        self.dirty.update(range(self.cursor.y, self.lines))
-        super(DiffScreen, self).insert_lines(*args)
-
-    def delete_lines(self, *args):
-        self.dirty.update(range(self.cursor.y, self.lines))
-        super(DiffScreen, self).delete_lines(*args)
-
-    def insert_characters(self, *args):
-        self.dirty.add(self.cursor.y)
-        super(DiffScreen, self).insert_characters(*args)
-
-    def delete_characters(self, *args):
-        self.dirty.add(self.cursor.y)
-        super(DiffScreen, self).delete_characters(*args)
-
-    def erase_characters(self, *args):
-        self.dirty.add(self.cursor.y)
-        super(DiffScreen, self).erase_characters(*args)
-
-    def erase_in_line(self, *args):
-        self.dirty.add(self.cursor.y)
-        super(DiffScreen, self).erase_in_line(*args)
-
-    def erase_in_display(self, how=0):
-        if how == 0:
-            self.dirty.update(range(self.cursor.y + 1, self.lines))
-        elif how == 1:
-            self.dirty.update(range(self.cursor.y))
-        elif how == 2:
-            self.dirty.update(range(self.lines))
-
-        super(DiffScreen, self).erase_in_display(how)
-
-    def alignment_display(self):
-        self.dirty.update(range(self.lines))
-        super(DiffScreen, self).alignment_display()
+        super(DiffScreen, self).__init__(*args, **kwargs)
 
 
 History = namedtuple("History", "top bottom ratio size position")
 
 
-class HistoryScreen(DiffScreen):
-    """A :class:~`pyte.screens.DiffScreen` subclass, which keeps track
+class HistoryScreen(Screen):
+    """A :class:~`pyte.screens.Screen` subclass, which keeps track
     of screen history and allows pagination. This is not linux-specific,
     but still useful; see page 462 of VT520 User's Manual.
 
@@ -1215,7 +1186,7 @@ class HistoryScreen(DiffScreen):
 
     def index(self):
         """Overloaded to update top history with the removed lines."""
-        top, bottom = self.margins
+        top, bottom = self.margins or Margins(0, self.lines - 1)
 
         if self.cursor.y == bottom:
             self.history.top.append(self.buffer[top])
@@ -1224,7 +1195,7 @@ class HistoryScreen(DiffScreen):
 
     def reverse_index(self):
         """Overloaded to update bottom history with the removed lines."""
-        top, bottom = self.margins
+        top, bottom = self.margins or Margins(0, self.lines - 1)
 
         if self.cursor.y == top:
             self.history.bottom.append(self.buffer[bottom])
@@ -1241,16 +1212,16 @@ class HistoryScreen(DiffScreen):
             mid = min(len(self.history.top),
                       int(math.ceil(self.lines * self.history.ratio)))
 
-            self.history.bottom.extendleft([
-                self.buffer[line]
-                for line in range(self.lines - 1, self.lines - mid - 1, -1)])
+            self.history.bottom.extendleft(
+                self.buffer[y]
+                for y in range(self.lines - 1, self.lines - mid - 1, -1))
             self.history = self.history \
                 ._replace(position=self.history.position - self.lines)
 
-            for line in range(self.lines - 1, mid - 1, -1):
-                self.buffer[line] = self.buffer[line - mid]
-            for line in range(mid - 1, -1, -1):
-                self.buffer[line] = self.history.top.pop()
+            for y in range(self.lines - 1, mid - 1, -1):
+                self.buffer[y] = self.buffer[y - mid]
+            for y in range(mid - 1, -1, -1):
+                self.buffer[y] = self.history.top.pop()
 
             self.dirty = set(range(self.lines))
 
@@ -1260,14 +1231,14 @@ class HistoryScreen(DiffScreen):
             mid = min(len(self.history.bottom),
                       int(math.ceil(self.lines * self.history.ratio)))
 
-            self.history.top.extend([self.buffer[line] for line in range(mid)])
+            self.history.top.extend(self.buffer[y] for y in range(mid))
             self.history = self.history \
                 ._replace(position=self.history.position + self.lines)
 
-            for line in range(self.lines - mid):
-                self.buffer[line] = self.buffer[line + mid]
-            for line in range(self.lines - mid, self.lines):
-                self.buffer[line] = self.history.bottom.popleft()
+            for y in range(self.lines - mid):
+                self.buffer[y] = self.buffer[y + mid]
+            for y in range(self.lines - mid, self.lines):
+                self.buffer[y] = self.history.bottom.popleft()
 
             self.dirty = set(range(self.lines))
 
